﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Interactivity;
using FilteringXamGrid.Controls;
using FilteringXamGrid.Helpers;
using Infragistics.Controls.Grids;

namespace FilteringXamGrid.Behaviors
{

    public class XamGridFilterBehavior : Behavior<XamGrid>
    {
        private XamGrid Grid
        {
            get
            {
                return AssociatedObject;
            }
        }

        protected override void OnAttached()
        {
            base.OnAttached();
            Grid.Loaded += Grid_Loaded;
        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            Grid.Loaded -= Grid_Loaded;
        }

        void Grid_Loaded(object sender, RoutedEventArgs e)
        {
            if (Grid != null)
            {
                if (Grid.ItemsSource != null)
                {
                    Type typeOfCollection = (Grid.ItemsSource as System.Collections.ICollection).AsQueryable().ElementType;

                    var nullFilterOperandType = typeof(NullFilterOperand<>);
                    var notNullFilterOperandType = typeof(NotNullFilterOperand<>);

                    var nullType = nullFilterOperandType.MakeGenericType(typeOfCollection);
                    var notNullType = notNullFilterOperandType.MakeGenericType(typeOfCollection);

                    foreach (var column in Grid.Columns.OfType<CustomTemplateColumn>())
                    {
                        Grid.Columns.DataColumns[column.Key].FilterColumnSettings.RowFilterOperands.Add((FilterOperand)Activator.CreateInstance(nullType, column.FilterColumn));
                        Grid.Columns.DataColumns[column.Key].FilterColumnSettings.RowFilterOperands.Add((FilterOperand)Activator.CreateInstance(notNullType, column.FilterColumn));
                    }
                }
            }
        }
    }
}
