﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using System.Windows.Interactivity;
using Infragistics.Controls.Grids;

namespace FilteringXamGrid.Behaviors
{
    public class XamGridSelectedItemsBehavior : Behavior<XamGrid>
    {
        private XamGrid Grid
        {
            get
            {
                return AssociatedObject as XamGrid;
            }
        }

        public ObservableCollection<Object> SelectedItems
        {
            get { return (ObservableCollection<Object>)GetValue(SelectedItemsProperty); }
            set { SetValue(SelectedItemsProperty, value); }
        }

        public static readonly DependencyProperty SelectedItemsProperty = DependencyProperty.Register(
            "SelectedItems",
            typeof(ObservableCollection<Object>),
            typeof(XamGridSelectedItemsBehavior), null
            );


        protected override void OnAttached()
        {
            base.OnAttached();
            Grid.SelectedRowsCollectionChanged += GridOnSelectedItemsChanged;

        }

        protected override void OnDetaching()
        {
            base.OnDetaching();
            Grid.SelectedRowsCollectionChanged -= GridOnSelectedItemsChanged;
        }

        private void GridOnSelectedItemsChanged(object sender, SelectionCollectionChangedEventArgs<SelectedRowsCollection> selectionCollectionChangedEventArgs)
        {
            SelectedItems.Clear();
            foreach (var record in Grid.SelectionSettings.SelectedRows)
            {
                SelectedItems.Add(record.Data);
            }
        }
    }
}
