﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Infragistics;
using Infragistics.Controls.Grids;

namespace FilteringXamGrid.Helpers
{
    public class NullFilterOperand : FilterOperand
    {
        protected override string DefaultDisplayName
        {
            get
            {
                return "Blanks";
            }
        }

        public override bool RequiresFilteringInput
        {
            get
            {
                return false;
            }
        }
    }

    public class NotNullFilterOperand : FilterOperand
    {
        protected override string DefaultDisplayName
        {
            get
            {
                return "Not Blanks";
            }
        }

        public override bool RequiresFilteringInput
        {
            get
            {
                return false;
            }
        }
    }

    public class NullFilterOperand<T> : NullFilterOperand
    {
        private readonly string _fieldName;
        public NullFilterOperand(string fieldName)
        {
            _fieldName = fieldName;
        }

        public override System.Linq.Expressions.Expression FilteringExpression(object value)
        {
            var filterContext = new MyFilterContext<T>();

            System.Linq.Expressions.Expression expr = filterContext.MakeAStringEmptyExpression(_fieldName, null);

            return expr;
        }
    }

    public class NotNullFilterOperand<T> : NotNullFilterOperand
    {
        private readonly string _fieldName;
        public NotNullFilterOperand(string fieldName)
        {
            _fieldName = fieldName;
        }

        public override System.Linq.Expressions.Expression FilteringExpression(object value)
        {
            var filterContext = new MyFilterContext<T>();

            System.Linq.Expressions.Expression expr = filterContext.MakeAStringNotEmptyExpression(_fieldName, null);

            return expr;
        }
    }

    public class MyFilterContext<T> : FilterContext<T>
    {
        public MyFilterContext()
            : base(typeof(string), false, false, new CachedTypedInfo())
        {

        }

        public System.Linq.Expressions.Expression<Func<T, bool>>
                    MakeAStringEmptyExpression(string fieldName, string value)
        {
            return CreateStringEmptyExpression<T>(fieldName, value);
        }

        public System.Linq.Expressions.Expression<Func<T, bool>>
                      MakeAStringNotEmptyExpression(string fieldName, string value)
        {
            return CreateStringNotEmptyExpression<T>(fieldName, value);
        }
    }
}
