﻿// Type: Infragistics.Controls.Grids.TemplateColumn
// Assembly: InfragisticsWPF4.Controls.Grids.XamGrid.v12.2, Version=12.2.20122.2151, Culture=neutral, PublicKeyToken=7dd5c3163f2cd0cb
// Assembly location: C:\Program Files\Infragistics\NetAdvantage 2012.2\WPF\CLR4.0\Bin\SharedXAML\InfragisticsWPF4.Controls.Grids.XamGrid.v12.2.dll

using Infragistics;
using Infragistics.Controls.Grids.Primitives;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Data;

namespace Infragistics.Controls.Grids
{
  /// <summary>
  /// A column that allows the user to specify what content will get displayed via it's <see cref="P:Infragistics.Controls.Grids.TemplateColumn.ItemTemplate"/> property.
  /// 
  /// </summary>
  /// 
  /// <remarks>
  /// Note: the <see cref="T:Infragistics.Controls.Grids.TemplateColumn"/> does not recycle it's content.
  /// </remarks>
  public class TemplateColumn : EditableColumn
  {
    /// <summary>
    /// Identifies the <see cref="P:Infragistics.Controls.Grids.TemplateColumn.ItemTemplate"/> dependency property.
    /// 
    /// </summary>
    public static readonly DependencyProperty ItemTemplateProperty = DependencyProperty.Register("ItemTemplate", typeof (DataTemplate), typeof (TemplateColumn), new PropertyMetadata(new PropertyChangedCallback(TemplateColumn.ItemTemplateChanged)));
    /// <summary>
    /// Identifies the <see cref="P:Infragistics.Controls.Grids.TemplateColumn.EditorTemplate"/> dependency property.
    /// 
    /// </summary>
    public static readonly DependencyProperty EditorTemplateProperty = DependencyProperty.Register("EditorTemplate", typeof (DataTemplate), typeof (TemplateColumn), new PropertyMetadata(new PropertyChangedCallback(TemplateColumn.EditorTemplateChanged)));
    /// <summary>
    /// Identifies the <see cref="P:Infragistics.Controls.Grids.TemplateColumn.FilterItemTemplate"/> dependency property.
    /// 
    /// </summary>
    public static readonly DependencyProperty FilterItemTemplateProperty = DependencyProperty.Register("FilterItemTemplate", typeof (DataTemplate), typeof (TemplateColumn), new PropertyMetadata(new PropertyChangedCallback(TemplateColumn.FilterItemTemplateChanged)));
    /// <summary>
    /// Identifies the <see cref="P:Infragistics.Controls.Grids.TemplateColumn.FilterEditorTemplate"/> dependency property.
    /// 
    /// </summary>
    public static readonly DependencyProperty FilterEditorTemplateProperty = DependencyProperty.Register("FilterEditorTemplate", typeof (DataTemplate), typeof (TemplateColumn), new PropertyMetadata(new PropertyChangedCallback(TemplateColumn.FilterEditorTemplateChanged)));
    private string _filterMenuCustomFilterString;

    /// <summary>
    /// Gets / sets the <see cref="T:System.Windows.DataTemplate"/> that will be used to generate content for each <see cref="T:Infragistics.Controls.Grids.CellBase"/> of the <see cref="T:Infragistics.Controls.Grids.TemplateColumn"/>.
    /// 
    /// </summary>
    public DataTemplate ItemTemplate
    {
      get
      {
        return (DataTemplate) this.GetValue(TemplateColumn.ItemTemplateProperty);
      }
      set
      {
        this.SetValue(TemplateColumn.ItemTemplateProperty, (object) value);
      }
    }

    /// <summary>
    /// Gets/sets the Editor that will be displayed when a <see cref="T:Infragistics.Controls.Grids.Cell"/> in this <see cref="T:Infragistics.Controls.Grids.TemplateColumn"/> is in edit mode.
    /// 
    /// </summary>
    public DataTemplate EditorTemplate
    {
      get
      {
        return (DataTemplate) this.GetValue(TemplateColumn.EditorTemplateProperty);
      }
      set
      {
        this.SetValue(TemplateColumn.EditorTemplateProperty, (object) value);
      }
    }

    /// <summary>
    /// Gets / sets the <see cref="T:System.Windows.DataTemplate"/> that will be used to generate content for the <see cref="T:Infragistics.Controls.Grids.CellBase"/> of the <see cref="T:Infragistics.Controls.Grids.TemplateColumn"/> in the <see cref="T:Infragistics.Controls.Grids.Primitives.FilterRowCell"/>.
    /// 
    /// </summary>
    /// 
    /// <remarks>
    /// If this property is null then the <see cref="P:Infragistics.Controls.Grids.TemplateColumn.ItemTemplate"/> is used.
    /// 
    /// </remarks>
    public DataTemplate FilterItemTemplate
    {
      get
      {
        return (DataTemplate) this.GetValue(TemplateColumn.FilterItemTemplateProperty);
      }
      set
      {
        this.SetValue(TemplateColumn.FilterItemTemplateProperty, (object) value);
      }
    }

    /// <summary>
    /// Gets / sets the Editor that will be displayed when this <see cref="T:Infragistics.Controls.Grids.Primitives.FilterRowCell"/> in this <see cref="T:Infragistics.Controls.Grids.TemplateColumn"/> is in edit mode.
    /// 
    /// </summary>
    /// 
    /// <remarks>
    /// If this property is null then the <see cref="P:Infragistics.Controls.Grids.TemplateColumn.EditorTemplate"/> is used.
    /// 
    /// </remarks>
    public DataTemplate FilterEditorTemplate
    {
      get
      {
        return (DataTemplate) this.GetValue(TemplateColumn.FilterEditorTemplateProperty);
      }
      set
      {
        this.SetValue(TemplateColumn.FilterEditorTemplateProperty, (object) value);
      }
    }

    /// <summary>
    /// Gets the default string for the FilterMenu for the CustomFilter text
    /// 
    /// </summary>
    protected override string FilterMenuCustomFilterString
    {
      get
      {
        if (string.IsNullOrEmpty(this._filterMenuCustomFilterString))
          return base.FilterMenuCustomFilterString;
        else
          return SRGrid.GetString(this._filterMenuCustomFilterString);
      }
    }

    internal int EditorTemplateDirtyFlag { get; set; }

    /// <summary>
    /// Gets/Sets whether the <see cref="T:Infragistics.Controls.Grids.TemplateColumn"/> is sortable.
    /// 
    /// </summary>
    /// 
    /// <remarks>
    /// In order for a TemplateColumn to be sortable, it must have a Key that maps to a property on the underlying datasource.
    /// 
    /// </remarks>
    public override bool IsSortable
    {
      get
      {
        if (!base.IsSortable)
          return false;
        if (!(this.DataType != (Type) null))
          return this.SortComparer != null;
        else
          return true;
      }
      set
      {
        base.IsSortable = value;
      }
    }

    /// <summary>
    /// Resolves whether this <see cref="T:Infragistics.Controls.Grids.Column"/> supports editing.
    /// 
    /// </summary>
    protected internal override bool IsEditable
    {
      get
      {
        return !this.IsReadOnly;
      }
    }

    /// <summary>
    /// Resolves whether <see cref="T:Infragistics.Controls.Grids.Cell"/> objects can be recycled and shared amongst columns of the same type.
    /// 
    /// </summary>
    /// 
    /// <remarks>
    /// Note: If false, then the Cells generated by this Column's ContentProvider will be
    ///             shared between other column's of this specific type.
    /// 
    /// </remarks>
    protected internal override bool UniqueColumnContent
    {
      get
      {
        return true;
      }
    }

    /// <summary>
    /// This property isn't supported on this <see cref="T:Infragistics.Controls.Grids.Column"/>.
    /// 
    /// </summary>
    [EditorBrowsable(EditorBrowsableState.Never)]
    public new IValueConverter EditorValueConverter
    {
      get
      {
        return (IValueConverter) this.GetValue(EditableColumn.EditorValueConverterProperty);
      }
      set
      {
        this.SetValue(EditableColumn.EditorValueConverterProperty, (object) value);
      }
    }

    /// <summary>
    /// This property isn't supported on this <see cref="T:Infragistics.Controls.Grids.Column"/>.
    /// 
    /// </summary>
    [EditorBrowsable(EditorBrowsableState.Never)]
    public new object EditorValueConverterParameter
    {
      get
      {
        return this.GetValue(EditableColumn.EditorValueConverterParameterProperty);
      }
      set
      {
        this.SetValue(EditableColumn.EditorValueConverterParameterProperty, value);
      }
    }

    /// <summary>
    /// This property isn't supported on this <see cref="T:Infragistics.Controls.Grids.Column"/>.
    /// 
    /// </summary>
    [EditorBrowsable(EditorBrowsableState.Never)]
    public new IValueConverter ValueConverter
    {
      get
      {
        return (IValueConverter) this.GetValue(Column.ValueConverterProperty);
      }
      set
      {
        this.SetValue(Column.ValueConverterProperty, (object) value);
      }
    }

    /// <summary>
    /// This property isn't supported on this <see cref="T:Infragistics.Controls.Grids.Column"/>.
    /// 
    /// </summary>
    [EditorBrowsable(EditorBrowsableState.Never)]
    public new object ValueConverterParameter
    {
      get
      {
        return this.GetValue(Column.ValueConverterParameterProperty);
      }
      set
      {
        this.SetValue(Column.ValueConverterParameterProperty, value);
      }
    }

    /// <summary>
    /// Determines if the <see cref="T:Infragistics.Controls.Grids.Column"/> should use the ReadOnly flag on a property, to determine if it can enter edit mode.
    /// 
    /// </summary>
    protected internal override bool UseReadOnlyFlag
    {
      get
      {
        return false;
      }
    }

    static TemplateColumn()
    {
    }

    private static void ItemTemplateChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
    {
      TemplateColumn templateColumn = (TemplateColumn) obj;
      if (templateColumn.ColumnLayout == null || templateColumn.ColumnLayout.Grid == null || !templateColumn.ColumnLayout.Grid.IsLoaded)
        return;
      templateColumn.ColumnLayout.Grid.ResetPanelRows(true);
    }

    private static void EditorTemplateChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
    {
      ++((TemplateColumn) obj).EditorTemplateDirtyFlag;
    }

    private static void FilterItemTemplateChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
    {
      ((DependencyObjectNotifier) obj).OnPropertyChanged("FilterItemTemplate");
    }

    private static void FilterEditorTemplateChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
    {
      ((DependencyObjectNotifier) obj).OnPropertyChanged("FilterEditorTemplate");
    }

    /// <summary>
    /// Generates a new <see cref="T:Infragistics.Controls.Grids.Primitives.TemplateColumnContentProvider"/> that will be used to generate conent for <see cref="T:Infragistics.Controls.Grids.Cell"/> objects for this <see cref="T:Infragistics.Controls.Grids.Column"/>.
    /// 
    /// </summary>
    /// 
    /// <returns/>
    protected internal override ColumnContentProviderBase GenerateContentProvider()
    {
      return (ColumnContentProviderBase) new TemplateColumnContentProvider();
    }

    /// <summary>
    /// Fills the <see cref="T:Infragistics.Controls.Grids.FilterOperandCollection"/> with the operands that the column expects as filter values.
    /// 
    /// </summary>
    /// <param name="availableFilters"/>
    protected internal override void FillAvailableFilters(FilterOperandCollection availableFilters)
    {
      base.FillAvailableFilters(availableFilters);
      if (this.ColumnLayout == null || this.ColumnLayout.Grid == null || (this.ColumnLayout.Grid.FilterIcons == null || !(this.DataType != (Type) null)))
        return;
      if (this.DataType == typeof (string))
      {
        FilterOperandCollection operandCollection1 = availableFilters;
        StartsWithOperand startsWithOperand1 = new StartsWithOperand();
        startsWithOperand1.XamWebGrid = this.ColumnLayout.Grid;
        StartsWithOperand startsWithOperand2 = startsWithOperand1;
        operandCollection1.Add((FilterOperand) startsWithOperand2);
        FilterOperandCollection operandCollection2 = availableFilters;
        ContainsOperand containsOperand1 = new ContainsOperand();
        containsOperand1.XamWebGrid = this.ColumnLayout.Grid;
        ContainsOperand containsOperand2 = containsOperand1;
        operandCollection2.Add((FilterOperand) containsOperand2);
        FilterOperandCollection operandCollection3 = availableFilters;
        EndsWithOperand endsWithOperand1 = new EndsWithOperand();
        endsWithOperand1.XamWebGrid = this.ColumnLayout.Grid;
        EndsWithOperand endsWithOperand2 = endsWithOperand1;
        operandCollection3.Add((FilterOperand) endsWithOperand2);
        FilterOperandCollection operandCollection4 = availableFilters;
        DoesNotContainOperand notContainOperand1 = new DoesNotContainOperand();
        notContainOperand1.XamWebGrid = this.ColumnLayout.Grid;
        DoesNotContainOperand notContainOperand2 = notContainOperand1;
        operandCollection4.Add((FilterOperand) notContainOperand2);
        FilterOperandCollection operandCollection5 = availableFilters;
        DoesNotStartWithOperand startWithOperand1 = new DoesNotStartWithOperand();
        startWithOperand1.XamWebGrid = this.ColumnLayout.Grid;
        DoesNotStartWithOperand startWithOperand2 = startWithOperand1;
        operandCollection5.Add((FilterOperand) startWithOperand2);
        FilterOperandCollection operandCollection6 = availableFilters;
        DoesNotEndWithOperand notEndWithOperand1 = new DoesNotEndWithOperand();
        notEndWithOperand1.XamWebGrid = this.ColumnLayout.Grid;
        DoesNotEndWithOperand notEndWithOperand2 = notEndWithOperand1;
        operandCollection6.Add((FilterOperand) notEndWithOperand2);
      }
      else
      {
        if (!this.DataType.IsValueType)
          return;
        FilterOperandCollection operandCollection1 = availableFilters;
        GreaterThanOperand greaterThanOperand1 = new GreaterThanOperand();
        greaterThanOperand1.XamWebGrid = this.ColumnLayout.Grid;
        GreaterThanOperand greaterThanOperand2 = greaterThanOperand1;
        operandCollection1.Add((FilterOperand) greaterThanOperand2);
        FilterOperandCollection operandCollection2 = availableFilters;
        GreaterThanOrEqualOperand thanOrEqualOperand1 = new GreaterThanOrEqualOperand();
        thanOrEqualOperand1.XamWebGrid = this.ColumnLayout.Grid;
        GreaterThanOrEqualOperand thanOrEqualOperand2 = thanOrEqualOperand1;
        operandCollection2.Add((FilterOperand) thanOrEqualOperand2);
        FilterOperandCollection operandCollection3 = availableFilters;
        LessThanOperand lessThanOperand1 = new LessThanOperand();
        lessThanOperand1.XamWebGrid = this.ColumnLayout.Grid;
        LessThanOperand lessThanOperand2 = lessThanOperand1;
        operandCollection3.Add((FilterOperand) lessThanOperand2);
        FilterOperandCollection operandCollection4 = availableFilters;
        LessThanOrEqualOperand thanOrEqualOperand3 = new LessThanOrEqualOperand();
        thanOrEqualOperand3.XamWebGrid = this.ColumnLayout.Grid;
        LessThanOrEqualOperand thanOrEqualOperand4 = thanOrEqualOperand3;
        operandCollection4.Add((FilterOperand) thanOrEqualOperand4);
      }
    }

    /// <summary>
    /// Fills the inputted list with options for the FilterMenu control.
    /// 
    /// </summary>
    /// <param name="list"/>
    protected internal override void FillFilterMenuOptions(List<FilterMenuTrackingObject> list)
    {
      base.FillFilterMenuOptions(list);
      if (!(this.DataType != (Type) null) || this.ColumnLayout == null || (this.ColumnLayout.Grid == null || list == null) || list.Count <= 0)
        return;
      list = list[0].Children;
      if (this.DataType == typeof (string))
      {
        list.Add(new FilterMenuTrackingObject()
        {
          IsSeparator = true
        });
        List<FilterMenuTrackingObject> list1 = list;
        StartsWithOperand startsWithOperand = new StartsWithOperand();
        startsWithOperand.DisplayName = SRGrid.GetString("StartsWithEllipsis");
        FilterMenuTrackingObject menuTrackingObject1 = new FilterMenuTrackingObject((FilterOperand) startsWithOperand);
        list1.Add(menuTrackingObject1);
        List<FilterMenuTrackingObject> list2 = list;
        EndsWithOperand endsWithOperand = new EndsWithOperand();
        endsWithOperand.DisplayName = SRGrid.GetString("EndsWithEllipsis");
        FilterMenuTrackingObject menuTrackingObject2 = new FilterMenuTrackingObject((FilterOperand) endsWithOperand);
        list2.Add(menuTrackingObject2);
        list.Add(new FilterMenuTrackingObject()
        {
          IsSeparator = true
        });
        List<FilterMenuTrackingObject> list3 = list;
        ContainsOperand containsOperand = new ContainsOperand();
        containsOperand.DisplayName = SRGrid.GetString("ContainsEllipsis");
        FilterMenuTrackingObject menuTrackingObject3 = new FilterMenuTrackingObject((FilterOperand) containsOperand);
        list3.Add(menuTrackingObject3);
        List<FilterMenuTrackingObject> list4 = list;
        DoesNotContainOperand notContainOperand = new DoesNotContainOperand();
        notContainOperand.DisplayName = SRGrid.GetString("DoesNotContainEllipsis");
        FilterMenuTrackingObject menuTrackingObject4 = new FilterMenuTrackingObject((FilterOperand) notContainOperand);
        list4.Add(menuTrackingObject4);
        list.Add(new FilterMenuTrackingObject()
        {
          IsSeparator = true
        });
        List<FilterMenuTrackingObject> list5 = list;
        DoesNotStartWithOperand startWithOperand = new DoesNotStartWithOperand();
        startWithOperand.DisplayName = SRGrid.GetString("DoesNotStartWithEllipsis");
        FilterMenuTrackingObject menuTrackingObject5 = new FilterMenuTrackingObject((FilterOperand) startWithOperand);
        list5.Add(menuTrackingObject5);
        List<FilterMenuTrackingObject> list6 = list;
        DoesNotEndWithOperand notEndWithOperand = new DoesNotEndWithOperand();
        notEndWithOperand.DisplayName = SRGrid.GetString("DoesNotEndWithEllipsis");
        FilterMenuTrackingObject menuTrackingObject6 = new FilterMenuTrackingObject((FilterOperand) notEndWithOperand);
        list6.Add(menuTrackingObject6);
      }
      else
      {
        if (!this.DataType.IsPrimitive && !FilterRow.IsNullableValueType(this.DataType) || (!(this.DataType != typeof (bool)) || !(this.DataType != typeof (bool?))) || (!(this.DataType != typeof (DateTime)) || !(this.DataType != typeof (DateTime?))))
          return;
        list.Add(new FilterMenuTrackingObject()
        {
          IsSeparator = true
        });
        List<FilterMenuTrackingObject> list1 = list;
        GreaterThanOperand greaterThanOperand = new GreaterThanOperand();
        greaterThanOperand.DisplayName = SRGrid.GetString("GreaterThanEllipsis");
        FilterMenuTrackingObject menuTrackingObject1 = new FilterMenuTrackingObject((FilterOperand) greaterThanOperand);
        list1.Add(menuTrackingObject1);
        List<FilterMenuTrackingObject> list2 = list;
        GreaterThanOrEqualOperand thanOrEqualOperand1 = new GreaterThanOrEqualOperand();
        thanOrEqualOperand1.DisplayName = SRGrid.GetString("GreaterThanOrEqualEllipsis");
        FilterMenuTrackingObject menuTrackingObject2 = new FilterMenuTrackingObject((FilterOperand) thanOrEqualOperand1);
        list2.Add(menuTrackingObject2);
        List<FilterMenuTrackingObject> list3 = list;
        LessThanOperand lessThanOperand = new LessThanOperand();
        lessThanOperand.DisplayName = SRGrid.GetString("LessThanEllipsis");
        FilterMenuTrackingObject menuTrackingObject3 = new FilterMenuTrackingObject((FilterOperand) lessThanOperand);
        list3.Add(menuTrackingObject3);
        List<FilterMenuTrackingObject> list4 = list;
        LessThanOrEqualOperand thanOrEqualOperand2 = new LessThanOrEqualOperand();
        thanOrEqualOperand2.DisplayName = SRGrid.GetString("LessThanOrEqualEllipsis");
        FilterMenuTrackingObject menuTrackingObject4 = new FilterMenuTrackingObject((FilterOperand) thanOrEqualOperand2);
        list4.Add(menuTrackingObject4);
        list.Add(new FilterMenuTrackingObject()
        {
          Label = SRGrid.GetString("BetweenEllipsis"),
          FilterOperands = {
            (FilterOperand) new GreaterThanOrEqualOperand(),
            (FilterOperand) new LessThanOrEqualOperand()
          }
        });
      }
    }

    /// <summary>
    /// Raised when the DataType of the <see cref="T:Infragistics.Controls.Grids.ColumnBase"/> is changed.
    /// 
    /// </summary>
    protected override void OnDataTypeChanged()
    {
      if (this.DataType == typeof (string))
        this._filterMenuCustomFilterString = "StringFilters";
      else if ((this.DataType.IsPrimitive || FilterRow.IsNullableValueType(this.DataType)) && (this.DataType != typeof (bool) && this.DataType != typeof (bool?)) && (this.DataType != typeof (DateTime) && this.DataType != typeof (DateTime?)))
        this._filterMenuCustomFilterString = "NumberFilters";
      else if (this.DataType == typeof (bool) || this.DataType == typeof (bool?))
        this._filterMenuCustomFilterString = "BoolFilters";
      base.OnDataTypeChanged();
    }
  }
}
