﻿// Type: Infragistics.FilterContext`1
// Assembly: InfragisticsWPF4.DataManager.v12.2, Version=12.2.20122.2151, Culture=neutral, PublicKeyToken=7dd5c3163f2cd0cb
// Assembly location: C:\Program Files\Infragistics\NetAdvantage 2012.2\WPF\CLR4.0\Bin\SharedXAML\InfragisticsWPF4.DataManager.v12.2.dll

using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Linq.Expressions;
using System.Reflection;

namespace Infragistics
{
  /// <summary>
  /// A <see cref="T:Infragistics.FilterContext"/> object typed to a particular object.
  /// 
  /// </summary>
  /// <typeparam name="TDataObject">The type of the object that will be processed against.</typeparam>
  public class FilterContext<TDataObject> : FilterContext
  {
    protected static readonly MethodInfo EqualsMethod = typeof (object).GetMethod("Equals", new Type[1]
    {
      typeof (object)
    });

    static FilterContext()
    {
    }

    /// <summary>
    /// Initializes a new instance of the <see cref="T:Infragistics.FilterContext"/> class.
    /// 
    /// </summary>
    /// <param name="fieldDataType"/><param name="caseSensitive"/><param name="fromDateColumn"/><param name="cti"/>
    public FilterContext(Type fieldDataType, bool caseSensitive, bool fromDateColumn, CachedTypedInfo cti)
    {
      this.CaseSensitive = caseSensitive;
      this.FieldDataType = fieldDataType;
      this.FromDateColumn = fromDateColumn;
      this.CachedTypedInfo = cti;
    }

    internal override Expression And(Expression left, Expression right)
    {
      return (Expression) this.AndAlsoExpression<TDataObject>((Expression<Func<TDataObject, bool>>) left, (Expression<Func<TDataObject, bool>>) right);
    }

    internal override Expression Or(Expression left, Expression right)
    {
      return (Expression) this.OrElseExpression<TDataObject>((Expression<Func<TDataObject, bool>>) left, (Expression<Func<TDataObject, bool>>) right);
    }

    /// <summary>
    /// Combines two <see cref="T:System.Linq.Expressions.Expression"/> objects with a OR Expression.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="left"/><param name="right"/>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which is the combination of the two inputs.
    /// </returns>
    protected internal Expression<Func<TDataObject, bool>> OrElseExpression<TDataObjectType>(Expression<Func<TDataObject, bool>> left, Expression<Func<TDataObject, bool>> right)
    {
      InvocationExpression invocationExpression = Expression.Invoke((Expression) right, System.Linq.Enumerable.Cast<Expression>((IEnumerable) left.Parameters));
      return Expression.Lambda<Func<TDataObject, bool>>((Expression) Expression.OrElse(left.Body, (Expression) invocationExpression), (IEnumerable<ParameterExpression>) left.Parameters);
    }

    /// <summary>
    /// Combines two <see cref="T:System.Linq.Expressions.Expression"/> objects with a AND Expression.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="left"/><param name="right"/>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which is the combination of the two inputs.
    /// </returns>
    protected internal Expression<Func<TDataObject, bool>> AndAlsoExpression<TDataObjectType>(Expression<Func<TDataObject, bool>> left, Expression<Func<TDataObject, bool>> right)
    {
      InvocationExpression invocationExpression = Expression.Invoke((Expression) right, System.Linq.Enumerable.Cast<Expression>((IEnumerable) left.Parameters));
      return Expression.Lambda<Func<TDataObject, bool>>((Expression) Expression.AndAlso(left.Body, (Expression) invocationExpression), (IEnumerable<ParameterExpression>) left.Parameters);
    }

    /// <summary>
    /// Combines two <see cref="T:System.Linq.Expressions.Expression"/> objects with a XOR Expression.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="left"/><param name="right"/>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which is the combination of the two inputs.
    /// </returns>
    protected internal Expression<Func<TDataObject, bool>> ExclusiveOrExpression<TDataObjectType>(Expression<Func<TDataObject, bool>> left, Expression<Func<TDataObject, bool>> right)
    {
      InvocationExpression invocationExpression = Expression.Invoke((Expression) right, System.Linq.Enumerable.Cast<Expression>((IEnumerable) left.Parameters));
      return Expression.Lambda<Func<TDataObject, bool>>((Expression) Expression.ExclusiveOr(left.Body, (Expression) invocationExpression), (IEnumerable<ParameterExpression>) left.Parameters);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for equality.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDateTimeTimeExcludedEqualsExpression<TDataObjectType>(string fieldName, object value)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObjectType), "parameter");
      Expression left1 = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression right1 = (Expression) Expression.Constant(value, this.FieldDataType);
      Expression expression;
      if (value != null)
      {
        Expression left2 = (Expression) Expression.GreaterThanOrEqual(left1, right1);
        DateTime dateTime = (DateTime) value;
        dateTime = dateTime.AddDays(1.0);
        Expression right2 = (Expression) Expression.Constant((object) dateTime, this.FieldDataType);
        Expression right3 = (Expression) Expression.LessThan(left1, right2);
        expression = (Expression) Expression.AndAlso(left2, right3);
      }
      else
        expression = (Expression) Expression.Equal(left1, right1);
      if (!typeof (TDataObject).IsValueType)
        expression = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression);
      return Expression.Lambda<Func<TDataObject, bool>>(expression, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for equality.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateEqualsExpression<TDataObjectType>(string fieldName, object value)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObjectType), "parameter");
      Expression expression1 = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression right1 = (Expression) Expression.Constant(value, this.FieldDataType);
      Expression expression2;
      if (this.FromDateColumn && value != null)
      {
        Expression left = (Expression) Expression.GreaterThanOrEqual(expression1, right1);
        DateTime dateTime = (DateTime) value;
        dateTime = dateTime.AddDays(1.0);
        Expression right2 = (Expression) Expression.Constant((object) dateTime, this.FieldDataType);
        Expression right3 = (Expression) Expression.LessThan(expression1, right2);
        expression2 = (Expression) Expression.AndAlso(left, right3);
      }
      else if (!expression1.Type.IsValueType)
        expression2 = (Expression) Expression.Condition((Expression) Expression.Equal(expression1, (Expression) Expression.Constant((object) null, expression1.Type)), (Expression) Expression.Equal(expression1, right1), (Expression) Expression.Call(expression1, FilterContext<TDataObject>.EqualsMethod, new Expression[1]
        {
          right1
        }));
      else
        expression2 = (Expression) Expression.Equal(expression1, right1);
      if (!typeof (TDataObject).IsValueType)
        expression2 = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression2);
      return Expression.Lambda<Func<TDataObject, bool>>(expression2, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for inequality.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateNotEqualsExpression<TDataObjectType>(string fieldName, object value)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObjectType), "parameter");
      Expression expression1 = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression right1 = (Expression) Expression.Constant(value, this.FieldDataType);
      Expression expression2;
      if (this.FromDateColumn && value != null)
      {
        Expression left = (Expression) Expression.GreaterThanOrEqual(expression1, right1);
        DateTime dateTime = (DateTime) value;
        dateTime = dateTime.AddDays(1.0);
        Expression right2 = (Expression) Expression.Constant((object) dateTime, this.FieldDataType);
        Expression right3 = (Expression) Expression.LessThan(expression1, right2);
        expression2 = (Expression) Expression.Not((Expression) Expression.AndAlso(left, right3));
      }
      else if (!expression1.Type.IsValueType)
        expression2 = (Expression) Expression.Condition((Expression) Expression.Equal(expression1, (Expression) Expression.Constant((object) null, expression1.Type)), (Expression) Expression.NotEqual(expression1, right1), (Expression) Expression.Not((Expression) Expression.Call(expression1, FilterContext<TDataObject>.EqualsMethod, new Expression[1]
        {
          right1
        })));
      else
        expression2 = (Expression) Expression.NotEqual(expression1, right1);
      if (!typeof (TDataObject).IsValueType)
        expression2 = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression2);
      return Expression.Lambda<Func<TDataObject, bool>>(expression2, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for greater than the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateGreaterThanExpression<TDataObjectType>(string fieldName, object value)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObject), "parameter");
      Expression left = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression right1 = (Expression) Expression.Constant(value, this.FieldDataType);
      Expression expression;
      if (this.FromDateColumn && value != null)
      {
        Expression right2 = (Expression) Expression.Constant((object) ((DateTime) value).AddDays(1.0), this.FieldDataType);
        expression = (Expression) Expression.GreaterThanOrEqual(left, right2);
      }
      else
        expression = (Expression) Expression.GreaterThan(left, right1);
      if (!typeof (TDataObject).IsValueType)
        expression = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression);
      return Expression.Lambda<Func<TDataObject, bool>>(expression, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for greater than or equal the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateGreaterThanOrEqualsExpression<TDataObjectType>(string fieldName, object value)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObject), "parameter");
      Expression expression = (Expression) Expression.GreaterThanOrEqual(DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue()), (Expression) Expression.Constant(value, this.FieldDataType));
      if (!typeof (TDataObject).IsValueType)
        expression = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression);
      return Expression.Lambda<Func<TDataObject, bool>>(expression, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for less than the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateLessThanExpression<TDataObjectType>(string fieldName, object value)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObject), "parameter");
      Expression expression = (Expression) Expression.LessThan(DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue()), (Expression) Expression.Constant(value, this.FieldDataType));
      if (!typeof (TDataObject).IsValueType)
        expression = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression);
      return Expression.Lambda<Func<TDataObject, bool>>(expression, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for less than or equal the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateLessThanOrEqualsExpression<TDataObjectType>(string fieldName, object value)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObject), "parameter");
      Expression left = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression right1 = (Expression) Expression.Constant(value, this.FieldDataType);
      Expression expression;
      if (this.FromDateColumn && value != null)
      {
        Expression right2 = (Expression) Expression.Constant((object) ((DateTime) value).AddDays(1.0), this.FieldDataType);
        expression = (Expression) Expression.LessThan(left, right2);
      }
      else
        expression = (Expression) Expression.LessThanOrEqual(left, right1);
      if (!typeof (TDataObject).IsValueType)
        expression = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression);
      return Expression.Lambda<Func<TDataObject, bool>>(expression, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Builds an expression for analyzing string
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="booleanStringExpression">A <![CDATA[Expression<Func<string, bool>>]]> which will analyze a string.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringExpression<TDataObjectType>(string fieldName, Expression<Func<string, bool>> booleanStringExpression)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObject), "parameter");
      Expression expression1 = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression expression2 = (Expression) Expression.Invoke((Expression) booleanStringExpression, new Expression[1]
      {
        expression1
      });
      if (!typeof (TDataObject).IsValueType)
        expression2 = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression2);
      return Expression.Lambda<Func<TDataObject, bool>>(expression2, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive equals against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateEqualsCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass0<TDataObjectType> cDisplayClass0 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass0<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass0.value = value;
      // ISSUE: reference to a compiler-generated field
      if (!string.IsNullOrEmpty(cDisplayClass0.value))
      {
        FilterContext<TDataObject> filterContext = this;
        string fieldName1 = fieldName;
        ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
        // ISSUE: method reference
        // ISSUE: field reference
        // ISSUE: type reference
        Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.Equals)), new Expression[2]
        {
          (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass0), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass0<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass0<TDataObjectType>))),
          (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
        })), new ParameterExpression[1]
        {
          parameterExpression
        });
        return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
      }
      else
      {
        FilterContext<TDataObject> filterContext = this;
        string fieldName1 = fieldName;
        ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
        // ISSUE: method reference
        Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.Call((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.IsNullOrEmpty)), new Expression[1]
        {
          (Expression) parameterExpression
        }), new ParameterExpression[1]
        {
          parameterExpression
        });
        return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
      }
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive inequality against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateNotEqualsCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2<TDataObjectType> cDisplayClass2 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass2.value = value;
      // ISSUE: reference to a compiler-generated field
      if (!string.IsNullOrEmpty(cDisplayClass2.value))
      {
        FilterContext<TDataObject> filterContext = this;
        string fieldName1 = fieldName;
        ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
        // ISSUE: method reference
        // ISSUE: field reference
        // ISSUE: type reference
        Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Not((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.Equals)), new Expression[2]
        {
          (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass2), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2<TDataObjectType>))),
          (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
        }))), new ParameterExpression[1]
        {
          parameterExpression
        });
        return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
      }
      else
      {
        FilterContext<TDataObject> filterContext = this;
        string fieldName1 = fieldName;
        ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
        // ISSUE: method reference
        Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.Not((Expression) Expression.Call((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.IsNullOrEmpty)), new Expression[1]
        {
          (Expression) parameterExpression
        })), new ParameterExpression[1]
        {
          parameterExpression
        });
        return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
      }
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive starts with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStartsWithExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass4<TDataObjectType> cDisplayClass4 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass4<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass4.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.StartsWith)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass4), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass4<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass4<TDataObjectType>)))
      })), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive starts with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStartsWithCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass6<TDataObjectType> cDisplayClass6 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass6<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass6.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.StartsWith)), new Expression[2]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass6), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass6<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass6<TDataObjectType>))),
        (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
      })), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive ends with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateEndsWithExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass8<TDataObjectType> cDisplayClass8 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass8<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass8.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.EndsWith)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass8), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass8<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass8<TDataObjectType>)))
      })), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive ends with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateEndsWithCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassa<TDataObjectType> cDisplayClassa = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassa<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClassa.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.EndsWith)), new Expression[2]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClassa), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassa<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassa<TDataObjectType>))),
        (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
      })), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive contains against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateContainsExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassc<TDataObjectType> cDisplayClassc = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassc<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClassc.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.Contains)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClassc), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassc<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClassc<TDataObjectType>)))
      })), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive contains against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateContainsCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClasse<TDataObjectType> cDisplayClasse = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClasse<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClasse.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.GreaterThanOrEqual((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.IndexOf)), new Expression[2]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClasse), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClasse<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClasse<TDataObjectType>))),
        (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
      }), (Expression) Expression.Constant((object) 0, typeof (int)))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive does not contain against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDoesNotContainExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass10<TDataObjectType> cDisplayClass10 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass10<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass10.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Not((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.Contains)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass10), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass10<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass10<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive does not contain against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDoesNotContainCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass12<TDataObjectType> cDisplayClass12 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass12<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass12.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Equal((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.IndexOf)), new Expression[2]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass12), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass12<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass12<TDataObjectType>))),
        (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
      }), (Expression) Expression.Constant((object) -1, typeof (int)))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive greater than against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringGreaterThanExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass14<TDataObjectType> cDisplayClass14 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass14<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass14.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Equal((Expression) Expression.Constant((object) 1, typeof (int)), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.CompareTo)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass14), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass14<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass14<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive greater than against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringGreaterThanCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass16<TDataObjectType> cDisplayClass16 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass16<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass16.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Equal((Expression) Expression.Constant((object) 1, typeof (int)), (Expression) Expression.Call((Expression) Expression.Property((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.get_CurrentCultureIgnoreCase))), (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.Compare)), new Expression[2]
      {
        (Expression) parameterExpression,
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass16), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass16<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass16<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive greater than or equal against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringGreaterThanOrEqualExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass18<TDataObjectType> cDisplayClass18 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass18<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass18.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.NotEqual((Expression) Expression.Constant((object) -1, typeof (int)), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.CompareTo)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass18), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass18<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass18<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive greater than or equal against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringGreaterThanOrEqualCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1a<TDataObjectType> cDisplayClass1a = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1a<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass1a.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.NotEqual((Expression) Expression.Constant((object) -1, typeof (int)), (Expression) Expression.Call((Expression) Expression.Property((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.get_CurrentCultureIgnoreCase))), (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.Compare)), new Expression[2]
      {
        (Expression) parameterExpression,
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass1a), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1a<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1a<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive less than against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringLessThanExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1c<TDataObjectType> cDisplayClass1c = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1c<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass1c.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Equal((Expression) Expression.Constant((object) -1, typeof (int)), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.CompareTo)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass1c), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1c<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1c<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive less than against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringLessThanCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1e<TDataObjectType> cDisplayClass1e = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1e<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass1e.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Equal((Expression) Expression.Constant((object) -1, typeof (int)), (Expression) Expression.Call((Expression) Expression.Property((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.get_CurrentCultureIgnoreCase))), (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.Compare)), new Expression[2]
      {
        (Expression) parameterExpression,
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass1e), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1e<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass1e<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive less than or equal against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringLessThanOrEqualExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass20<TDataObjectType> cDisplayClass20 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass20<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass20.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.NotEqual((Expression) Expression.Constant((object) 1, typeof (int)), (Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.CompareTo)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass20), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass20<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass20<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive less than or equal against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringLessThanOrEqualCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass22<TDataObjectType> cDisplayClass22 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass22<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass22.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.NotEqual((Expression) Expression.Constant((object) 1, typeof (int)), (Expression) Expression.Call((Expression) Expression.Property((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.get_CurrentCultureIgnoreCase))), (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (StringComparer.Compare)), new Expression[2]
      {
        (Expression) parameterExpression,
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass22), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass22<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass22<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for empty strings values.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringEmptyExpression<TDataObjectType>(string fieldName, string value)
    {
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.Call((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.IsNullOrEmpty)), new Expression[1]
      {
        (Expression) parameterExpression
      }), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for nonempty strings values.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateStringNotEmptyExpression<TDataObjectType>(string fieldName, string value)
    {
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.Not((Expression) Expression.Call((Expression) null, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.IsNullOrEmpty)), new Expression[1]
      {
        (Expression) parameterExpression
      })), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive does not starts with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDoesNotStartWithExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass24<TDataObjectType> cDisplayClass24 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass24<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass24.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Not((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.StartsWith)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass24), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass24<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass24<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive does not starts with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDoesNotStartWithCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass26<TDataObjectType> cDisplayClass26 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass26<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass26.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Not((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.StartsWith)), new Expression[2]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass26), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass26<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass26<TDataObjectType>))),
        (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case sensitive does not ends with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDoesNotEndWithExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass28<TDataObjectType> cDisplayClass28 = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass28<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass28.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Not((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.EndsWith)), new Expression[1]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass28), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass28<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass28<TDataObjectType>)))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for case insensitive does not ends with against the inputted value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="value">The value that will be analyzed against.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDoesNotEndWithCaseInsensitiveExpression<TDataObjectType>(string fieldName, string value)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2a<TDataObjectType> cDisplayClass2a = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2a<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass2a.value = value;
      FilterContext<TDataObject> filterContext = this;
      string fieldName1 = fieldName;
      ParameterExpression parameterExpression = Expression.Parameter(typeof (string), "x");
      // ISSUE: method reference
      // ISSUE: field reference
      // ISSUE: type reference
      Expression<Func<string, bool>> booleanStringExpression = Expression.Lambda<Func<string, bool>>((Expression) Expression.OrElse((Expression) Expression.Equal((Expression) parameterExpression, (Expression) Expression.Constant((object) null, typeof (object))), (Expression) Expression.Not((Expression) Expression.Call((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (string.EndsWith)), new Expression[2]
      {
        (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass2a), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2a<TDataObjectType>.value), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2a<TDataObjectType>))),
        (Expression) Expression.Constant((object) StringComparison.CurrentCultureIgnoreCase, typeof (StringComparison))
      }))), new ParameterExpression[1]
      {
        parameterExpression
      });
      return filterContext.CreateStringExpression<TDataObject>(fieldName1, booleanStringExpression);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> based on all the terms in the <see cref="T:Infragistics.ConditionCollection"/>.
    /// 
    /// </summary>
    /// <param name="conditionGroup"/>
    /// <returns/>
    public override Expression CreateExpression(ConditionCollection conditionGroup)
    {
      Expression<Func<TDataObject, bool>> left = (Expression<Func<TDataObject, bool>>) null;
      if (conditionGroup.Count > 0)
      {
        left = (Expression<Func<TDataObject, bool>>) conditionGroup[0].GetCurrentExpression((FilterContext) this);
        for (int index = 1; index < conditionGroup.Count; ++index)
        {
          Expression<Func<TDataObject, bool>> right = (Expression<Func<TDataObject, bool>>) conditionGroup[index].GetCurrentExpression((FilterContext) this);
          left = conditionGroup.LogicalOperator != LogicalOperator.Or ? this.AndAlsoExpression<TDataObject>(left, right) : this.OrElseExpression<TDataObject>(left, right);
        }
      }
      return (Expression) left;
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> from the <see cref="T:Infragistics.RecordFilterCollection"/>
    /// </summary>
    /// <param name="conditionGroup"/>
    /// <returns/>
    public override Expression CreateExpression(RecordFilterCollection conditionGroup)
    {
      Expression<Func<TDataObject, bool>> left = (Expression<Func<TDataObject, bool>>) null;
      if (conditionGroup.Count > 0)
      {
        left = (Expression<Func<TDataObject, bool>>) conditionGroup[0].GetCurrentExpression();
        for (int index = 1; index < conditionGroup.Count; ++index)
        {
          Expression<Func<TDataObject, bool>> right = (Expression<Func<TDataObject, bool>>) conditionGroup[index].GetCurrentExpression();
          if (left == null)
            left = right;
          else if (right != null)
            left = conditionGroup.LogicalOperator != LogicalOperator.Or ? this.AndAlsoExpression<TDataObject>(left, right) : this.OrElseExpression<TDataObject>(left, right);
        }
      }
      return (Expression) left;
    }

    /// <summary>
    /// Creates an expression based on the <see cref="T:Infragistics.ComparisonOperator"/>.
    /// 
    /// </summary>
    /// <param name="fieldName"/><param name="op"/><param name="value"/>
    /// <returns/>
    public override Expression CreateExpression(string fieldName, ComparisonOperator op, object value)
    {
      if (!(this.FieldDataType == typeof (string)))
        return this.CreateObjectExpression(fieldName, op, value);
      if (this.CaseSensitive)
        return this.CreateCaseSensitiveStringExpression(fieldName, op, value);
      else
        return this.CreateCaseInsensitiveStringExpression(fieldName, op, value);
    }

    /// <summary>
    /// Creates a new <see cref="T:System.Linq.Expressions.Expression"/>
    /// </summary>
    /// <param name="fieldName"/><param name="op"/><param name="value"/><param name="caseSensitive"/>
    /// <returns/>
    public override Expression CreateExpression(string fieldName, ComparisonOperator op, object value, bool caseSensitive)
    {
      if (!(this.FieldDataType == typeof (string)))
        return this.CreateObjectExpression(fieldName, op, value);
      if (caseSensitive)
        return this.CreateCaseSensitiveStringExpression(fieldName, op, value);
      else
        return this.CreateCaseInsensitiveStringExpression(fieldName, op, value);
    }

    /// <summary>
    /// Returns an <see cref="T:System.Linq.Expressions.Expression"/> for a case insensitive string operation.
    /// 
    /// </summary>
    /// <param name="fieldName"/><param name="op"/><param name="value"/>
    /// <returns/>
    protected internal Expression CreateCaseInsensitiveStringExpression(string fieldName, ComparisonOperator op, object value)
    {
      string str = (string) value;
      Expression expression = (Expression) null;
      switch (op)
      {
        case ComparisonOperator.Equals:
          expression = (Expression) this.CreateEqualsCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.NotEquals:
          expression = (Expression) this.CreateNotEqualsCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.GreaterThan:
          expression = (Expression) this.CreateStringGreaterThanCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.GreaterThanOrEqual:
          expression = (Expression) this.CreateStringGreaterThanOrEqualCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.LessThan:
          expression = (Expression) this.CreateStringLessThanCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.LessThanOrEqual:
          expression = (Expression) this.CreateStringLessThanOrEqualCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.StartsWith:
          expression = (Expression) this.CreateStartsWithCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.DoesNotStartWith:
          expression = (Expression) this.CreateDoesNotStartWithCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.EndsWith:
          expression = (Expression) this.CreateEndsWithCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.DoesNotEndWith:
          expression = (Expression) this.CreateDoesNotEndWithCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.Contains:
          expression = (Expression) this.CreateContainsCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.DoesNotContain:
          expression = (Expression) this.CreateDoesNotContainCaseInsensitiveExpression<TDataObject>(fieldName, str);
          break;
      }
      return expression;
    }

    /// <summary>
    /// Returns an <see cref="T:System.Linq.Expressions.Expression"/> for a case sensitive string operation.
    /// 
    /// </summary>
    /// <param name="fieldName"/><param name="op"/><param name="value"/>
    /// <returns/>
    protected internal Expression CreateCaseSensitiveStringExpression(string fieldName, ComparisonOperator op, object value)
    {
      string str = (string) value;
      Expression expression = (Expression) null;
      switch (op)
      {
        case ComparisonOperator.Equals:
          expression = (Expression) this.CreateEqualsExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.NotEquals:
          expression = (Expression) this.CreateNotEqualsExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.GreaterThan:
          expression = (Expression) this.CreateStringGreaterThanExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.GreaterThanOrEqual:
          expression = (Expression) this.CreateStringGreaterThanOrEqualExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.LessThan:
          expression = (Expression) this.CreateStringLessThanExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.LessThanOrEqual:
          expression = (Expression) this.CreateStringLessThanOrEqualExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.StartsWith:
          expression = (Expression) this.CreateStartsWithExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.DoesNotStartWith:
          expression = (Expression) this.CreateDoesNotStartWithExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.EndsWith:
          expression = (Expression) this.CreateEndsWithExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.DoesNotEndWith:
          expression = (Expression) this.CreateDoesNotEndWithExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.Contains:
          expression = (Expression) this.CreateContainsExpression<TDataObject>(fieldName, str);
          break;
        case ComparisonOperator.DoesNotContain:
          expression = (Expression) this.CreateDoesNotContainExpression<TDataObject>(fieldName, str);
          break;
      }
      return expression;
    }

    /// <summary>
    /// Returns an <see cref="T:System.Linq.Expressions.Expression"/> for a object operation.
    /// 
    /// </summary>
    /// <param name="fieldName"/><param name="op"/><param name="value"/>
    /// <returns/>
    protected internal Expression CreateObjectExpression(string fieldName, ComparisonOperator op, object value)
    {
      Expression expression = (Expression) null;
      switch (op)
      {
        case ComparisonOperator.Equals:
          expression = (Expression) this.CreateEqualsExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.NotEquals:
          expression = (Expression) this.CreateNotEqualsExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.GreaterThan:
          expression = (Expression) this.CreateGreaterThanExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.GreaterThanOrEqual:
          expression = (Expression) this.CreateGreaterThanOrEqualsExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.LessThan:
          expression = (Expression) this.CreateLessThanExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.LessThanOrEqual:
          expression = (Expression) this.CreateLessThanOrEqualsExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.DateTimeAfter:
          expression = (Expression) this.CreateGreaterThanExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.DateTimeBefore:
          expression = (Expression) this.CreateLessThanExpression<TDataObject>(fieldName, value);
          break;
        case ComparisonOperator.DateTimeToday:
          expression = (Expression) this.CreateDateTimeTimeExcludedEqualsExpression<TDataObject>(fieldName, (object) DateTime.Today);
          break;
        case ComparisonOperator.DateTimeTomorrow:
          expression = (Expression) this.CreateDateTimeTimeExcludedEqualsExpression<TDataObject>(fieldName, (object) DateTime.Today.AddDays(1.0));
          break;
        case ComparisonOperator.DateTimeYesterday:
          expression = (Expression) this.CreateDateTimeTimeExcludedEqualsExpression<TDataObject>(fieldName, (object) DateTime.Today.AddDays(-1.0));
          break;
        case ComparisonOperator.DateTimeThisWeek:
          expression = (Expression) this.CreateThisWeekExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeNextWeek:
          expression = (Expression) this.CreateNextWeekExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeLastWeek:
          expression = (Expression) this.CreateLastWeekExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeThisMonth:
          expression = (Expression) this.CreateThisMonthExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeLastMonth:
          expression = (Expression) this.CreateLastMonthExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeNextMonth:
          expression = (Expression) this.CreateNextMonthExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeThisYear:
          expression = (Expression) this.CreateThisYearExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeLastYear:
          expression = (Expression) this.CreateLastYearExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeNextYear:
          expression = (Expression) this.CreateNextYearExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeYearToDate:
          expression = (Expression) this.CreateYearToDateExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeLastQuarter:
          expression = (Expression) this.CreateLastQuarterExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeThisQuarter:
          expression = (Expression) this.CreateThisQuarterExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeNextQuarter:
          expression = (Expression) this.CreateNextQuarterExpression<TDataObject>(fieldName);
          break;
        case ComparisonOperator.DateTimeJanuary:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 1);
          break;
        case ComparisonOperator.DateTimeFebruary:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 2);
          break;
        case ComparisonOperator.DateTimeMarch:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 3);
          break;
        case ComparisonOperator.DateTimeApril:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 4);
          break;
        case ComparisonOperator.DateTimeMay:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 5);
          break;
        case ComparisonOperator.DateTimeJune:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 6);
          break;
        case ComparisonOperator.DateTimeJuly:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 7);
          break;
        case ComparisonOperator.DateTimeAugust:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 8);
          break;
        case ComparisonOperator.DateTimeSeptember:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 9);
          break;
        case ComparisonOperator.DateTimeOctober:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 10);
          break;
        case ComparisonOperator.DateTimeNovember:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 11);
          break;
        case ComparisonOperator.DateTimeDecember:
          expression = (Expression) this.CreateMonthExpression<TDataObject>(fieldName, 12);
          break;
        case ComparisonOperator.DateTimeQuarter1:
          expression = (Expression) this.CreateQuarterYearIndependentExpression<TDataObject>(fieldName, 1);
          break;
        case ComparisonOperator.DateTimeQuarter2:
          expression = (Expression) this.CreateQuarterYearIndependentExpression<TDataObject>(fieldName, 2);
          break;
        case ComparisonOperator.DateTimeQuarter3:
          expression = (Expression) this.CreateQuarterYearIndependentExpression<TDataObject>(fieldName, 3);
          break;
        case ComparisonOperator.DateTimeQuarter4:
          expression = (Expression) this.CreateQuarterYearIndependentExpression<TDataObject>(fieldName, 4);
          break;
      }
      return expression;
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for this week.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateThisWeekExpression<TDataObjectType>(string fieldName)
    {
      DateTime includeStartDate = DateTime.Today.AddDays((double) (-(int) DateTime.Today.DayOfWeek + CultureInfo.CurrentCulture.DateTimeFormat.FirstDayOfWeek));
      includeStartDate = new DateTime(includeStartDate.Year, includeStartDate.Month, includeStartDate.Day);
      DateTime excludedEndDate = includeStartDate.AddDays(7.0);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for next week.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateNextWeekExpression<TDataObjectType>(string fieldName)
    {
      DateTime includeStartDate = DateTime.Today.AddDays((double) (-(int) DateTime.Today.DayOfWeek + CultureInfo.CurrentCulture.DateTimeFormat.FirstDayOfWeek));
      includeStartDate = new DateTime(includeStartDate.Year, includeStartDate.Month, includeStartDate.Day);
      includeStartDate = includeStartDate.AddDays(7.0);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, includeStartDate.AddDays(7.0));
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for last week.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateLastWeekExpression<TDataObjectType>(string fieldName)
    {
      DateTime includeStartDate = DateTime.Today.AddDays((double) (-(int) DateTime.Today.DayOfWeek + CultureInfo.CurrentCulture.DateTimeFormat.FirstDayOfWeek));
      includeStartDate = new DateTime(includeStartDate.Year, includeStartDate.Month, includeStartDate.Day);
      includeStartDate = includeStartDate.AddDays(-7.0);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, includeStartDate.AddDays(7.0));
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for this Month.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateThisMonthExpression<TDataObjectType>(string fieldName)
    {
      DateTime includeStartDate = new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1);
      DateTime dateTime = DateTime.Today.AddMonths(1);
      DateTime excludedEndDate = new DateTime(dateTime.Year, dateTime.Month, 1);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for next Month.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateNextMonthExpression<TDataObjectType>(string fieldName)
    {
      DateTime dateTime = DateTime.Today.AddMonths(1);
      DateTime includeStartDate = new DateTime(dateTime.Year, dateTime.Month, 1);
      dateTime = dateTime.AddMonths(1);
      DateTime excludedEndDate = new DateTime(dateTime.Year, dateTime.Month, 1);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for last Month.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateLastMonthExpression<TDataObjectType>(string fieldName)
    {
      DateTime excludedEndDate = new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1);
      DateTime dateTime = DateTime.Today.AddMonths(-1);
      DateTime includeStartDate = new DateTime(dateTime.Year, dateTime.Month, 1);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for this Year.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateThisYearExpression<TDataObjectType>(string fieldName)
    {
      DateTime includeStartDate = new DateTime(DateTime.Today.Year, 1, 1);
      DateTime excludedEndDate = new DateTime(DateTime.Today.Year + 1, 1, 1);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for next Year.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateNextYearExpression<TDataObjectType>(string fieldName)
    {
      DateTime includeStartDate = new DateTime(DateTime.Today.Year + 1, 1, 1);
      DateTime excludedEndDate = new DateTime(DateTime.Today.Year + 2, 1, 1);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for last Year.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateLastYearExpression<TDataObjectType>(string fieldName)
    {
      DateTime excludedEndDate = new DateTime(DateTime.Today.Year, 1, 1);
      DateTime includeStartDate = new DateTime(DateTime.Today.Year - 1, 1, 1);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for this Quarter.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateThisQuarterExpression<TDataObjectType>(string fieldName)
    {
      FilterContext<TDataObject>.QuarterTracking currentQuarterInfo = this.GetCurrentQuarterInfo(DateTime.Today);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, currentQuarterInfo.StartDate, currentQuarterInfo.EndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for next Quarter.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateNextQuarterExpression<TDataObjectType>(string fieldName)
    {
      FilterContext<TDataObject>.QuarterTracking nextQuarterInfo = this.GetNextQuarterInfo(DateTime.Today);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, nextQuarterInfo.StartDate, nextQuarterInfo.EndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for last Quarter.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateLastQuarterExpression<TDataObjectType>(string fieldName)
    {
      FilterContext<TDataObject>.QuarterTracking previousQuarterInfo = this.GetPreviousQuarterInfo(DateTime.Today);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, previousQuarterInfo.StartDate, previousQuarterInfo.EndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for year to date.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateYearToDateExpression<TDataObjectType>(string fieldName)
    {
      DateTime includeStartDate = new DateTime(DateTime.Today.Year, 1, 1);
      DateTime excludedEndDate = new DateTime(DateTime.Today.Year, DateTime.Today.Month, DateTime.Today.Day);
      excludedEndDate = excludedEndDate.AddDays(1.0);
      return this.CreateDateRangeExpression<TDataObject>(fieldName, includeStartDate, excludedEndDate);
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that evaluates for date ranges.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="includeStartDate"/><param name="excludedEndDate"/>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateDateRangeExpression<TDataObjectType>(string fieldName, DateTime includeStartDate, DateTime excludedEndDate)
    {
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObjectType), "parameter");
      Expression left1 = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression right1 = (Expression) Expression.Constant((object) includeStartDate, this.FieldDataType);
      Expression left2 = (Expression) Expression.GreaterThanOrEqual(left1, right1);
      Expression right2 = (Expression) Expression.Constant((object) excludedEndDate, this.FieldDataType);
      Expression right3 = (Expression) Expression.LessThan(left1, right2);
      Expression expression = (Expression) Expression.AndAlso(left2, right3);
      if (!typeof (TDataObject).IsValueType)
        expression = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression);
      return Expression.Lambda<Func<TDataObject, bool>>(expression, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that will evaluate an object's DateTime field for a particular month value.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName">The property that will be evaluated on.</param><param name="month">The int month value which will be filterd for.</param>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateMonthExpression<TDataObjectType>(string fieldName, int month)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2c<TDataObjectType> cDisplayClass2c = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2c<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass2c.month = month;
      Expression expression1;
      if (this.FieldDataType == typeof (DateTime))
      {
        ParameterExpression parameterExpression = Expression.Parameter(typeof (DateTime), "x");
        // ISSUE: method reference
        // ISSUE: field reference
        // ISSUE: type reference
        expression1 = (Expression) Expression.Lambda<Func<DateTime, bool>>((Expression) Expression.Equal((Expression) Expression.Property((Expression) parameterExpression, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (DateTime.get_Month))), (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass2c), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2c<TDataObjectType>.month), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2c<TDataObjectType>)))), new ParameterExpression[1]
        {
          parameterExpression
        });
      }
      else
      {
        ParameterExpression parameterExpression = Expression.Parameter(typeof (DateTime?), "x");
        // ISSUE: method reference
        // ISSUE: method reference
        // ISSUE: field reference
        // ISSUE: type reference
        expression1 = (Expression) Expression.Lambda<Func<DateTime?, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Convert((Expression) Expression.Constant((object) new DateTime?(), typeof (DateTime?)), typeof (DateTime?)), false, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (DateTime.op_Inequality))), (Expression) Expression.Equal((Expression) Expression.Property((Expression) Expression.Convert((Expression) parameterExpression, typeof (DateTime)), (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (DateTime.get_Month))), (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass2c), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2c<TDataObjectType>.month), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2c<TDataObjectType>))))), new ParameterExpression[1]
        {
          parameterExpression
        });
      }
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObject), "parameter");
      Expression expression2 = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression expression3 = (Expression) Expression.Invoke(expression1, new Expression[1]
      {
        expression2
      });
      if (!typeof (TDataObject).IsValueType)
        expression3 = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression3);
      return Expression.Lambda<Func<TDataObject, bool>>(expression3, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    /// <summary>
    /// Creates an <see cref="T:System.Linq.Expressions.Expression"/> that will evaluate an object's DateTime field for a particular quarter.
    /// 
    /// </summary>
    /// <typeparam name="TDataObjectType">The object type that the <see cref="T:System.Linq.Expressions.Expression"/> will be applied over.</typeparam><param name="fieldName"/><param name="quarter"/>
    /// <returns>
    /// An <see cref="T:System.Linq.Expressions.Expression"/> which will evaluate for the condition.
    /// </returns>
    protected internal virtual Expression<Func<TDataObject, bool>> CreateQuarterYearIndependentExpression<TDataObjectType>(string fieldName, int quarter)
    {
      // ISSUE: object of a compiler-generated type is created
      // ISSUE: variable of a compiler-generated type
      FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2e<TDataObjectType> cDisplayClass2e = new FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2e<TDataObjectType>();
      // ISSUE: reference to a compiler-generated field
      cDisplayClass2e.quarter = quarter;
      // ISSUE: reference to a compiler-generated field
      cDisplayClass2e.\u003C\u003E4__this = this;
      Expression expression1;
      if (this.FieldDataType == typeof (DateTime))
      {
        ParameterExpression parameterExpression = Expression.Parameter(typeof (DateTime), "x");
        // ISSUE: method reference
        // ISSUE: type reference
        // ISSUE: field reference
        // ISSUE: type reference
        expression1 = (Expression) Expression.Lambda<Func<DateTime, bool>>((Expression) Expression.Call((Expression) Expression.Constant((object) this, typeof (FilterContext<TDataObject>)), (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (FilterContext<TDataObject>.IsDateValueInQuarter), __typeref (FilterContext<TDataObject>)), new Expression[2]
        {
          (Expression) parameterExpression,
          (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass2e), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2e<TDataObjectType>.quarter), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2e<TDataObjectType>)))
        }), new ParameterExpression[1]
        {
          parameterExpression
        });
      }
      else
      {
        ParameterExpression parameterExpression = Expression.Parameter(typeof (DateTime?), "x");
        // ISSUE: method reference
        // ISSUE: method reference
        // ISSUE: type reference
        // ISSUE: field reference
        // ISSUE: type reference
        expression1 = (Expression) Expression.Lambda<Func<DateTime?, bool>>((Expression) Expression.AndAlso((Expression) Expression.NotEqual((Expression) parameterExpression, (Expression) Expression.Convert((Expression) Expression.Constant((object) new DateTime?(), typeof (DateTime?)), typeof (DateTime?)), false, (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (DateTime.op_Inequality))), (Expression) Expression.Call((Expression) Expression.Constant((object) this, typeof (FilterContext<TDataObject>)), (MethodInfo) MethodBase.GetMethodFromHandle(__methodref (FilterContext<TDataObject>.IsDateValueInQuarter), __typeref (FilterContext<TDataObject>)), new Expression[2]
        {
          (Expression) Expression.Convert((Expression) parameterExpression, typeof (DateTime)),
          (Expression) Expression.Field((Expression) Expression.Constant((object) cDisplayClass2e), FieldInfo.GetFieldFromHandle(__fieldref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2e<TDataObjectType>.quarter), __typeref (FilterContext<TDataObject>.\u003C\u003Ec__DisplayClass2e<TDataObjectType>)))
        })), new ParameterExpression[1]
        {
          parameterExpression
        });
      }
      ParameterExpression paramExpression = Expression.Parameter(typeof (TDataObject), "parameter");
      Expression expression2 = DataManagerBase.BuildPropertyExpressionFromPropertyName(fieldName, paramExpression, this.CachedTypedInfo, this.FieldDataType, this.GetDefaultValue());
      Expression expression3 = (Expression) Expression.Invoke(expression1, new Expression[1]
      {
        expression2
      });
      if (!typeof (TDataObject).IsValueType)
        expression3 = (Expression) Expression.Condition((Expression) Expression.Equal((Expression) paramExpression, (Expression) Expression.Constant((object) null, typeof (TDataObject))), (Expression) Expression.Constant((object) false), expression3);
      return Expression.Lambda<Func<TDataObject, bool>>(expression3, new ParameterExpression[1]
      {
        paramExpression
      });
    }

    internal bool IsDateValueInQuarter(DateTime value, int quarter)
    {
      int month1 = 1;
      int month2 = 1;
      int year1 = 1;
      int year2 = 1;
      switch (quarter)
      {
        case 1:
          month1 = 1;
          month2 = 4;
          year2 = year1 = value.Year;
          break;
        case 2:
          month1 = 4;
          month2 = 7;
          year2 = year1 = value.Year;
          break;
        case 3:
          month1 = 7;
          month2 = 10;
          year2 = year1 = value.Year;
          break;
        case 4:
          month1 = 10;
          month2 = 1;
          year2 = (year1 = value.Year) + 1;
          break;
      }
      DateTime dateTime1 = new DateTime(year1, month1, 1);
      DateTime dateTime2 = new DateTime(year2, month2, 1);
      if (value >= dateTime1)
        return value < dateTime2;
      else
        return false;
    }

    internal FilterContext<TDataObject>.QuarterTracking GetCurrentQuarterInfo(DateTime date)
    {
      int month = date.Month;
      int year = date.Year;
      FilterContext<TDataObject>.QuarterTracking quarterTracking = new FilterContext<TDataObject>.QuarterTracking();
      if (month <= 3)
      {
        quarterTracking.StartDate = new DateTime(year, 1, 1);
        quarterTracking.EndDate = new DateTime(year, 4, 1);
      }
      else if (month <= 6)
      {
        quarterTracking.StartDate = new DateTime(year, 4, 1);
        quarterTracking.EndDate = new DateTime(year, 7, 1);
      }
      else if (month <= 9)
      {
        quarterTracking.StartDate = new DateTime(year, 7, 1);
        quarterTracking.EndDate = new DateTime(year, 10, 1);
      }
      else
      {
        quarterTracking.StartDate = new DateTime(year, 10, 1);
        quarterTracking.EndDate = new DateTime(year + 1, 1, 1);
      }
      return quarterTracking;
    }

    internal FilterContext<TDataObject>.QuarterTracking GetNextQuarterInfo(DateTime date)
    {
      int month = date.Month;
      int year = date.Year;
      FilterContext<TDataObject>.QuarterTracking quarterTracking = new FilterContext<TDataObject>.QuarterTracking();
      if (month <= 3)
      {
        quarterTracking.StartDate = new DateTime(year, 4, 1);
        quarterTracking.EndDate = new DateTime(year, 7, 1);
      }
      else if (month <= 6)
      {
        quarterTracking.StartDate = new DateTime(year, 7, 1);
        quarterTracking.EndDate = new DateTime(year, 10, 1);
      }
      else if (month <= 9)
      {
        quarterTracking.StartDate = new DateTime(year, 10, 1);
        quarterTracking.EndDate = new DateTime(year + 1, 1, 1);
      }
      else
      {
        quarterTracking.StartDate = new DateTime(year + 1, 1, 1);
        quarterTracking.EndDate = new DateTime(year + 1, 4, 1);
      }
      return quarterTracking;
    }

    internal FilterContext<TDataObject>.QuarterTracking GetPreviousQuarterInfo(DateTime date)
    {
      int month = date.Month;
      int year = date.Year;
      FilterContext<TDataObject>.QuarterTracking quarterTracking = new FilterContext<TDataObject>.QuarterTracking();
      if (month <= 3)
      {
        quarterTracking.StartDate = new DateTime(year - 1, 10, 1);
        quarterTracking.EndDate = new DateTime(year, 1, 1);
      }
      else if (month <= 6)
      {
        quarterTracking.StartDate = new DateTime(year, 1, 1);
        quarterTracking.EndDate = new DateTime(year, 4, 1);
      }
      else if (month <= 9)
      {
        quarterTracking.StartDate = new DateTime(year, 4, 1);
        quarterTracking.EndDate = new DateTime(year, 7, 1);
      }
      else
      {
        quarterTracking.StartDate = new DateTime(year, 7, 1);
        quarterTracking.EndDate = new DateTime(year, 10, 1);
      }
      return quarterTracking;
    }

    private object GetDefaultValue()
    {
      if (!this.FieldDataType.IsValueType)
        return (object) null;
      else
        return Activator.CreateInstance(this.FieldDataType);
    }

    internal class QuarterTracking
    {
      public DateTime StartDate { get; set; }

      public DateTime EndDate { get; set; }
    }
  }
}
